@@ -73,6 +73,11 @@ class TableMetaParams:
     primary_key: [str] = field(default_factory=list)
     sorting_key: [str] = field(default_factory=list)
     indexes: [IndexDefinition] = field(default_factory=list)
+    # Using a skip index with FINAL queries may lead to incorrect results in some cases
+    # so Clickhouse added a `use_skip_indexes_if_final` setting which is false by default.
+    # Here we switch it on to make use of skip indexes as in normal situations it is safe.
+    # You can switch it off by using the following meta parameter with a False value.
+    use_skip_indexes_if_final: bool = True
 
 
 class ClickhouseCubeBackend(CubeBackend):
@@ -261,7 +266,10 @@ class ClickhouseCubeBackend(CubeBackend):
         # ensure MAX([]) is None not 0
         # but this does not seem to be compatible with automatic use of projections, so we
         # might need to turn it off in the future
-        settings_part = ", ".join(f"{k} = {v}" for k, v in self.default_settings.items())
+        applied_settings = {**self.default_settings}
+        if final and meta.use_skip_indexes_if_final:
+            applied_settings["use_skip_indexes_if_final"] = 1
+        settings_part = ", ".join(f"{k} = {v}" for k, v in applied_settings.items())
         if settings_part:
             text += f" SETTINGS {settings_part}"
         return text, params, fields, matview
